/******************************************************************************/
/*                                                                            */
/*	vr3 - version 3 results file viewer				      */
/*                                                                            */
/* Written by:	Peter Vaughan                                                 */
/*                                                                            */
/* 17/11/2001 - Modified to read v3.11 results files                          */
/******************************************************************************/

#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
 
#define RESULTSFILE "results.txt"

#define SCALEINCR 0.2
#define SCALETOTAL 70			/* must be multiple of 10 */


extern char *optarg;

double scaleincr = SCALEINCR;
int scaletotal = SCALETOTAL;
int scaleres;

pline(int n, char *s)
{
	int j;
	for(j=0; j<n; printf("%s",s), j++);
}

main(int argc, char **argv)
{
	FILE *p;
	struct stat buf;
	char buff[1000];
	int *totals;
	double pcent, maxpcent = 0.0, scale;
	int i, j, t, maxscale = 0, maxt = 0, rpa, count = 0;
	char resultsfile[128];
	int parm;

	strcpy(resultsfile, RESULTSFILE);

	while( (parm = getopt(argc, argv, "f:p:s:h")) != -1)
	switch(parm)
	{
		case 'h':
			printf("Usage: vr3 [-f resultsfile] [-p percent] [-s scale] [-h]\n");
			printf(" -f name of results file (default results.txt)\n");
			printf(" -p vertical %%age scale (default 0.2)\n");
			printf(" -s horizontal totals scale multiple of 10 (default 70)\n");
			printf(" -h this message\n");
			exit(1);
		case 'f':
			strcpy(resultsfile, optarg);
			break;

		case 'p':
			scaleincr = strtod(optarg, (char **)NULL);
			if(scaleincr == 0.0)
			{
				printf("Invalid -p parameter: default used\n");
				scaleincr = SCALEINCR;
			}
			printf("%s, %lf\n", optarg, scaleincr);
			break;

		case 's':
			scaletotal = atol(optarg);
			break;

		default:
			printf("Invalid arg: -f, -p, -s or -h only\n");
			exit(2);
	}


	scaletotal = abs(scaletotal / 10) * 10;
	scaleres = scaletotal / 10;
	if((p = fopen(resultsfile, "r")) == NULL)
	{
		printf("Cannot open results file %s\n", resultsfile);
		exit(1);
	}
	stat(resultsfile, &buf);
	/* count number of results */
	while (fgets(buff, 1000, p) != NULL) 
		if(sscanf(buff, "%lf, (%*ld  particles)",&pcent) == 1)
		{
			count++;
			if(pcent > maxpcent) maxpcent = pcent;
		}
	rewind(p);
	totals = (int *)calloc(((maxpcent+1.0)/scaleincr), sizeof(int));

	/* total each % bucket */
	while (fgets(buff, 1000, p) != NULL)
	{
		if(sscanf(buff, "%lf, (%*ld particles)",&pcent) == 1)
		{
			t = (int)(pcent/scaleincr);
			totals[t]++;
			if (totals[t] > maxscale) maxscale = totals[t];
			if(t > maxt) maxt = t;
		}
	}
	fclose(p);


	printf("\n================================\n");
	printf("Muon crunching version 3 results\n");
	printf("================================\n");
	printf("Results file '%s', filesize %d bytes\n\n",
			resultsfile, buf.st_size);
	printf("Results processed: %d, max %%age: %lf%%\n\n",
			count, maxpcent);

	rpa = 0;
	i = maxscale;
	while(i > 0)
	{
		i -= scaletotal;
		rpa++;
	}
	drawscale(rpa, 1);
	scale = 0.0;
	for (i=0; i<=maxt+1; i++)
	{
		printf("%2.2lf |", scale);
		pline(totals[i] / rpa,"*");
		printf("[%d]\n", totals[i]);
		scale += scaleincr;
	}
	drawscale(rpa, 0);

}

		
/*
/* scales
*/

drawscale(int rpa, int top)
{

	int i;

	if(!top) printf(" %%age "), pline(scaletotal,"-"), printf("\n");
	else printf("\t\t\t\tNumber of Results\n");
	if(rpa > 1 || scaletotal > 99)
	{
		pline(6, " ");
		for (i=1; i<=scaleres; i++)  pline(9, " "), printf("%1.0d",(i * rpa ) / 10);
		printf("\n");
	}
		
	pline(6, " ");
	
	for (i=1; i<=scaleres; i++)  pline(9, " "), printf("%1.1d",
		(((i * rpa * 10) - ((i * rpa / 10) * 100)) / 10));
	printf("\n");
	pline(6, " ");
	for (i=1; i<=scaleres; i++)  pline(9, " "), printf("0");
	printf("\n");

	if(top) printf(" %%age "), pline(scaletotal,"-"), printf("\n");
	else printf("\t\t\t\tNumber of Results\n");
}
